// Copyright (c) 2009-2016 The Regents of the University of Michigan
// This file is part of the HOOMD-blue project, released under the BSD 3-Clause License.

// Include the defined classes that are to be exported to python
#include "IntegratorHPMC.h"
#include "IntegratorHPMCMono.h"
#include "IntegratorHPMCMonoImplicit.h"
#include "ComputeFreeVolume.h"

#include "ShapeSphere.h"
#include "ShapeConvexPolygon.h"
#include "ShapePolyhedron.h"
#include "ShapeConvexPolyhedron.h"
#include "ShapeSpheropolyhedron.h"
#include "ShapeSpheropolygon.h"
#include "ShapeSimplePolygon.h"
#include "ShapeEllipsoid.h"
#include "ShapeFacetedSphere.h"
#include "ShapeSphinx.h"
#include "AnalyzerSDF.h"
#include "ShapeUnion.h"

#include "ExternalField.h"
#include "ExternalFieldWall.h"
#include "ExternalFieldLattice.h"
#include "ExternalFieldComposite.h"

#include "UpdaterExternalFieldWall.h"
#include "UpdaterRemoveDrift.h"
#include "UpdaterMuVT.h"
#include "UpdaterMuVTImplicit.h"

#ifdef ENABLE_CUDA
#include "IntegratorHPMCMonoGPU.h"
#include "IntegratorHPMCMonoImplicitGPU.h"
#include "ComputeFreeVolumeGPU.h"
#endif

// Include boost.python to do the exporting
#include <boost/python.hpp>

using namespace boost::python;
using namespace hpmc;

using namespace hpmc::detail;

namespace hpmc
{

//! Export the base HPMCMono integrators
void export_convex_spheropolyhedron8()
    {
    export_IntegratorHPMCMono< ShapeSpheropolyhedron<8> >("IntegratorHPMCMonoSpheropolyhedron8");
    export_IntegratorHPMCMonoImplicit< ShapeSpheropolyhedron<8> >("IntegratorHPMCMonoImplicitSpheropolyhedron8");
    export_ComputeFreeVolume< ShapeSpheropolyhedron<8> >("ComputeFreeVolumeSpheropolyhedron8");
    export_AnalyzerSDF< ShapeSpheropolyhedron<8> >("AnalyzerSDFSpheropolyhedron8");
    export_UpdaterMuVT< ShapeSpheropolyhedron<8> >("UpdaterMuVTSpheropolyhedron8");
    export_UpdaterMuVTImplicit< ShapeSpheropolyhedron<8> >("UpdaterMuVTImplicitSpheropolyhedron8");

    export_ExternalFieldInterface<ShapeSpheropolyhedron<8> >("ExternalFieldSpheropolyhedron8");
    export_LatticeField<ShapeSpheropolyhedron<8> >("ExternalFieldLatticeSpheropolyhedron8");
    export_ExternalFieldComposite<ShapeSpheropolyhedron<8> >("ExternalFieldCompositeSpheropolyhedron8");
    export_RemoveDriftUpdater<ShapeSpheropolyhedron<8> >("RemoveDriftUpdaterSpheropolyhedron8");
    // export_ExternalFieldWall<ShapeSpheropolyhedron<8> >("WallSpheropolyhedron8");
    // export_UpdaterExternalFieldWall<ShapeSpheropolyhedron<8> >("UpdaterExternalFieldWallSpheropolyhedron8");

    #ifdef ENABLE_CUDA

    export_IntegratorHPMCMonoGPU< ShapeSpheropolyhedron<8> >("IntegratorHPMCMonoGPUSpheropolyhedron8");
    export_IntegratorHPMCMonoImplicitGPU< ShapeSpheropolyhedron<8> >("IntegratorHPMCMonoImplicitGPUSpheropolyhedron8");
    export_ComputeFreeVolumeGPU< ShapeSpheropolyhedron<8> >("ComputeFreeVolumeGPUSpheropolyhedron8");

    #endif
    }

}
